@@ -13,7 +13,7 @@
 
 namespace Kate
 {
-TextRange::TextRange(TextBuffer &buffer, KTextEditor::Range range, InsertBehaviors insertBehavior, EmptyBehavior emptyBehavior)
+TextRange::TextRange(TextBuffer *buffer, KTextEditor::Range range, InsertBehaviors insertBehavior, EmptyBehavior emptyBehavior)
     : m_buffer(buffer)
     , m_start(buffer, this, range.start(), (insertBehavior & ExpandLeft) ? Kate::TextCursor::StayOnInsert : Kate::TextCursor::MoveOnInsert)
     , m_end(buffer, this, range.end(), (insertBehavior & ExpandRight) ? Kate::TextCursor::MoveOnInsert : Kate::TextCursor::StayOnInsert)
@@ -23,45 +23,46 @@ TextRange::TextRange(TextBuffer &buffer, KTextEditor::Range range, InsertBehavio
     , m_attributeOnlyForViews(false)
     , m_invalidateIfEmpty(emptyBehavior == InvalidateIfEmpty)
 {
-    // remember this range in buffer
-    m_buffer.m_ranges.insert(this);
-
     // check if range now invalid, there can happen no feedback, as m_feedback == 0
     // only place where KTextEditor::LineRange::invalid() for old range makes sense, as we were yet not registered!
     checkValidity();
 
     // Add the range if it is multiline
     if (spansMultipleBlocks()) {
-        m_buffer.addMultilineRange(this);
+        m_buffer->addMultilineRange(this);
     }
 }
 
 TextRange::~TextRange()
 {
+    if (!m_buffer) {
+        return;
+    }
     // reset feedback, don't want feedback during destruction
+    const bool hadFeedBack = m_feedback != nullptr;
+    const bool hadDynamicAttr = m_attribute
+        && (m_attribute->dynamicAttribute(KTextEditor::Attribute::ActivateCaretIn) || m_attribute->dynamicAttribute(KTextEditor::Attribute::ActivateMouseIn));
+    const bool notifyDeletion = hadFeedBack || hadDynamicAttr;
     m_feedback = nullptr;
 
     // remove range from cached multiline ranges
     const auto lineRange = toLineRange();
     if (lineRange.isValid() && spansMultipleBlocks()) {
-        m_buffer.removeMultilineRange(this);
+        m_buffer->removeMultilineRange(this);
     }
 
-    // remove this range from the buffer
-    m_buffer.m_ranges.remove(this);
-
     // trigger update, if we have attribute
     // notify right view
     // here we can ignore feedback, even with feedback, we want none if the range is deleted!
-    if (m_attribute) {
-        m_buffer.notifyAboutRangeChange(m_view, lineRange, true /* we have a attribute */);
+    if (m_attribute || notifyDeletion) {
+        m_buffer->notifyAboutRangeChange(m_view, lineRange, /*needsRepaint=*/m_attribute != nullptr, /*deletedRange=*/notifyDeletion ? this : nullptr);
     }
 }
 
 void TextRange::setInsertBehaviors(InsertBehaviors _insertBehaviors)
 {
     // nothing to do?
-    if (_insertBehaviors == insertBehaviors()) {
+    if (_insertBehaviors == insertBehaviors() || !m_buffer) {
         return;
     }
 
@@ -71,7 +72,7 @@ void TextRange::setInsertBehaviors(InsertBehaviors _insertBehaviors)
 
     // notify world
     if (m_attribute || m_feedback) {
-        m_buffer.notifyAboutRangeChange(m_view, toLineRange(), true /* we have a attribute */);
+        m_buffer->notifyAboutRangeChange(m_view, toLineRange(), true /* we have a attribute */);
     }
 }
 
@@ -100,16 +101,13 @@ void TextRange::setEmptyBehavior(EmptyBehavior emptyBehavior)
     // remember value
     m_invalidateIfEmpty = (emptyBehavior == InvalidateIfEmpty);
 
-    // invalidate range?
-    if (endInternal() <= startInternal()) {
-        setRange(KTextEditor::Range::invalid());
-    }
+    checkValidity();
 }
 
 void TextRange::setRange(KTextEditor::Range range)
 {
     // avoid work if nothing changed!
-    if (range == toRange()) {
+    if (!m_buffer || range == toRange()) {
         return;
     }
 
@@ -124,9 +122,9 @@ void TextRange::setRange(KTextEditor::Range range)
     const bool newSpansMultipleBlocks = spansMultipleBlocks();
     if (oldSpannedMultipleBlocks != newSpansMultipleBlocks) {
         if (oldSpannedMultipleBlocks) {
-            m_buffer.removeMultilineRange(this);
+            m_buffer->removeMultilineRange(this);
         } else {
-            m_buffer.addMultilineRange(this);
+            m_buffer->addMultilineRange(this);
         }
     }
 
@@ -152,7 +150,7 @@ void TextRange::setRange(KTextEditor::Range range)
 
     // notify buffer about attribute change, it will propagate the changes
     // notify right view
-    m_buffer.notifyAboutRangeChange(m_view, {startLineMin, endLineMax}, m_attribute);
+    m_buffer->notifyAboutRangeChange(m_view, {startLineMin, endLineMax}, m_attribute);
 
     // perhaps need to notify stuff!
     if (m_feedback) {
@@ -187,6 +185,7 @@ void TextRange::checkValidity(bool notifyAboutChange)
 
     KTextEditor::Cursor end(m_end.lineInternal(), m_end.columnInternal());
     KTextEditor::Cursor start(m_start.lineInternal(), m_start.columnInternal());
+
     // check if any cursor is invalid or the range is zero size and it should be invalidated then
     if (!start.isValid() || !end.isValid() || (m_invalidateIfEmpty && end <= start)) {
         m_start.setPosition(-1, -1);
@@ -200,8 +199,8 @@ void TextRange::checkValidity(bool notifyAboutChange)
     }
 
     // perhaps need to notify stuff!
-    if (notifyAboutChange && m_feedback) {
-        m_buffer.notifyAboutRangeChange(m_view, toLineRange(), false /* attribute not interesting here */);
+    if (notifyAboutChange && m_feedback && m_buffer) {
+        m_buffer->notifyAboutRangeChange(m_view, toLineRange(), false /* attribute not interesting here */);
 
         // do this last: may delete this range
         if (!toRange().isValid()) {
@@ -215,7 +214,7 @@ void TextRange::checkValidity(bool notifyAboutChange)
 void TextRange::setView(KTextEditor::View *view)
 {
     // nothing changes, nop
-    if (view == m_view) {
+    if (view == m_view || !m_buffer) {
         return;
     }
 
@@ -225,38 +224,53 @@ void TextRange::setView(KTextEditor::View *view)
     // notify buffer about attribute change, it will propagate the changes
     // notify all views (can be optimized later)
     if (m_attribute || m_feedback) {
-        m_buffer.notifyAboutRangeChange(nullptr, toLineRange(), m_attribute);
+        m_buffer->notifyAboutRangeChange(nullptr, toLineRange(), m_attribute);
     }
 }
 
 void TextRange::setAttribute(KTextEditor::Attribute::Ptr attribute)
 {
     // nothing changes, nop, only pointer compare
-    if (attribute == m_attribute) {
+    if (attribute == m_attribute || !m_buffer) {
         return;
     }
 
     // remember the new attribute
+    const bool hadDynamicAttr = m_attribute
+        && (m_attribute->dynamicAttribute(KTextEditor::Attribute::ActivateCaretIn) || m_attribute->dynamicAttribute(KTextEditor::Attribute::ActivateMouseIn));
+
     m_attribute = attribute;
 
+    const bool stillHasDynAttr = m_attribute
+        && (m_attribute->dynamicAttribute(KTextEditor::Attribute::ActivateCaretIn) || m_attribute->dynamicAttribute(KTextEditor::Attribute::ActivateMouseIn));
+    const bool notifyDeletion = hadDynamicAttr && !stillHasDynAttr;
+
     // notify buffer about attribute change, it will propagate the changes
     // notify right view
-    m_buffer.notifyAboutRangeChange(m_view, toLineRange(), true /* even for nullptr attribute, we had before one => repaint */);
+    // if dyn attr was cleared, then notify about deleted range now because if this range is deleted
+    // its deletion will not be notified because it will have no dyn attr.
+    m_buffer->notifyAboutRangeChange(m_view,
+                                     toLineRange(),
+                                     true /* even for nullptr attribute, we had before one => repaint */,
+                                     notifyDeletion ? this : nullptr);
 }
 
 void TextRange::setFeedback(KTextEditor::MovingRangeFeedback *feedback)
 {
     // nothing changes, nop
-    if (feedback == m_feedback) {
+    if (feedback == m_feedback || !m_buffer) {
         return;
     }
 
     // remember the new feedback object
+    const bool feedbackCleared = m_feedback && !feedback;
     m_feedback = feedback;
 
     // notify buffer about feedback change, it will propagate the changes
     // notify right view
-    m_buffer.notifyAboutRangeChange(m_view, toLineRange(), m_attribute);
+    // if feedback was set to null, then notify about deleted range now because if this range is deleted
+    // its deletion will not be notified because it has no feedback.
+    m_buffer->notifyAboutRangeChange(m_view, toLineRange(), m_attribute, feedbackCleared ? this : nullptr);
 }
 
 void TextRange::setAttributeOnlyForViews(bool onlyForViews)
@@ -268,7 +282,7 @@ void TextRange::setAttributeOnlyForViews(bool onlyForViews)
 void TextRange::setZDepth(qreal zDepth)
 {
     // nothing changes, nop
-    if (zDepth == m_zDepth) {
+    if (zDepth == m_zDepth || !m_buffer) {
         return;
     }
 
@@ -277,13 +291,13 @@ void TextRange::setZDepth(qreal zDepth)
 
     // notify buffer about attribute change, it will propagate the changes
     if (m_attribute) {
-        m_buffer.notifyAboutRangeChange(m_view, toLineRange(), m_attribute);
+        m_buffer->notifyAboutRangeChange(m_view, toLineRange(), m_attribute);
     }
 }
 
 KTextEditor::Document *Kate::TextRange::document() const
 {
-    return m_buffer.document();
+    return m_buffer ? m_buffer->document() : nullptr;
 }
 
 }
