@@ -47,7 +47,7 @@ void KateLineLayoutMap::relayoutLines(int startRealLine, int endRealLine)
     }
 }
 
-void KateLineLayoutMap::slotEditDone(int fromLine, int toLine, int shiftAmount, std::vector<KateTextLayout> &textLayouts)
+void KateLineLayoutMap::slotEditDone(KateRenderer *renderer, int fromLine, int toLine, int shiftAmount, std::vector<KateTextLayout> &textLayouts)
 {
     auto start = std::lower_bound(m_lineLayouts.begin(), m_lineLayouts.end(), LineLayoutPair(fromLine, nullptr), lessThan);
     auto end = std::upper_bound(start, m_lineLayouts.end(), LineLayoutPair(toLine, nullptr), lessThan);
@@ -55,7 +55,7 @@ void KateLineLayoutMap::slotEditDone(int fromLine, int toLine, int shiftAmount,
     if (shiftAmount != 0) {
         for (auto it = end; it != m_lineLayouts.end(); ++it) {
             (*it).first += shiftAmount;
-            (*it).second->setLine((*it).second->line() + shiftAmount);
+            (*it).second->setLine(renderer->folding(), (*it).second->line() + shiftAmount);
         }
 
         for (auto it = start; it != end; ++it) {
@@ -122,7 +122,7 @@ void KateLayoutCache::updateViewCache(const KTextEditor::Cursor startPos, int ne
     if (wrap()) {
         if (KateLineLayout *l = line(realLine)) {
             Q_ASSERT(l->isValid());
-            Q_ASSERT(l->length() >= startPos.column() || m_renderer->view()->wrapCursor());
+            Q_ASSERT(m_renderer->doc()->lineLength(l->line()) >= startPos.column() || m_renderer->view()->wrapCursor());
             bool found = false;
             for (; _viewLine < l->viewLineCount(); ++_viewLine) {
                 const KateTextLayout &t = l->viewLine(_viewLine);
@@ -218,15 +218,12 @@ KateLineLayout *KateLayoutCache::line(int realLine, int virtualLine)
             l->setVirtualLine(virtualLine);
         }
 
+        const Kate::TextLine textLine = acceptDirtyLayouts() ? m_renderer->doc()->plainKateTextLine(l->line()) : m_renderer->doc()->kateTextLine(l->line());
+
         if (l->layout().lineCount() <= 0) {
-            l->usePlainTextLine = acceptDirtyLayouts();
-            l->textLine(!acceptDirtyLayouts());
-            m_renderer->layoutLine(l, wrap() ? m_viewWidth : -1, enableLayoutCache);
+            m_renderer->layoutLine(textLine, l, wrap() ? m_viewWidth : -1, enableLayoutCache);
         } else if (l->layoutDirty && !acceptDirtyLayouts()) {
-            // reset textline
-            l->usePlainTextLine = false;
-            l->textLine(true);
-            m_renderer->layoutLine(l, wrap() ? m_viewWidth : -1, enableLayoutCache);
+            m_renderer->layoutLine(textLine, l, wrap() ? m_viewWidth : -1, enableLayoutCache);
         }
 
         Q_ASSERT(l->layout().lineCount() > 0 && (!l->layoutDirty || acceptDirtyLayouts()));
@@ -238,16 +235,12 @@ KateLineLayout *KateLayoutCache::line(int realLine, int virtualLine)
         return nullptr;
     }
 
-    KateLineLayout *l = new KateLineLayout(*m_renderer);
-    l->setLine(realLine, virtualLine);
+    KateLineLayout *l = new KateLineLayout;
+    l->setLine(m_renderer->folding(), realLine, virtualLine);
 
-    // Mark it dirty, because it may not have the syntax highlighting applied
-    // mark this here, to allow layoutLine to use plainLines...
-    if (acceptDirtyLayouts()) {
-        l->usePlainTextLine = true;
-    }
-
-    m_renderer->layoutLine(l, wrap() ? m_viewWidth : -1, enableLayoutCache);
+    // because it may not have the syntax highlighting applied, allow layoutLine to use plainLines...
+    const Kate::TextLine textLine = acceptDirtyLayouts() ? m_renderer->doc()->plainKateTextLine(l->line()) : m_renderer->doc()->kateTextLine(l->line());
+    m_renderer->layoutLine(textLine, l, wrap() ? m_viewWidth : -1, enableLayoutCache);
     Q_ASSERT(l->isValid());
 
     if (acceptDirtyLayouts()) {
@@ -289,11 +282,6 @@ KTextEditor::Cursor KateLayoutCache::viewCacheStart() const
     return !m_textLayouts.empty() ? m_textLayouts.front().start() : KTextEditor::Cursor();
 }
 
-KTextEditor::Cursor KateLayoutCache::viewCacheEnd() const
-{
-    return !m_textLayouts.empty() ? m_textLayouts.back().end() : KTextEditor::Cursor();
-}
-
 int KateLayoutCache::viewWidth() const
 {
     return m_viewWidth;
@@ -393,8 +381,10 @@ int KateLayoutCache::displayViewLine(const KTextEditor::Cursor virtualCursor, bo
     }
     ret += viewLine(realCursor);
 
-    if (limitToVisible && (ret < 0 || ret > limit)) {
+    if (limitToVisible && ret < 0) {
         return -1;
+    } else if (limitToVisible && ret > limit) {
+        return -2;
     }
 
     return ret;
@@ -431,22 +421,22 @@ void KateLayoutCache::viewCacheDebugOutput() const
 
 void KateLayoutCache::wrapLine(KTextEditor::Document *, const KTextEditor::Cursor position)
 {
-    m_lineLayouts.slotEditDone(position.line(), position.line() + 1, 1, m_textLayouts);
+    m_lineLayouts.slotEditDone(m_renderer, position.line(), position.line() + 1, 1, m_textLayouts);
 }
 
 void KateLayoutCache::unwrapLine(KTextEditor::Document *, int line)
 {
-    m_lineLayouts.slotEditDone(line - 1, line, -1, m_textLayouts);
+    m_lineLayouts.slotEditDone(m_renderer, line - 1, line, -1, m_textLayouts);
 }
 
 void KateLayoutCache::insertText(KTextEditor::Document *, const KTextEditor::Cursor position, const QString &)
 {
-    m_lineLayouts.slotEditDone(position.line(), position.line(), 0, m_textLayouts);
+    m_lineLayouts.slotEditDone(m_renderer, position.line(), position.line(), 0, m_textLayouts);
 }
 
 void KateLayoutCache::removeText(KTextEditor::Document *, KTextEditor::Range range, const QString &)
 {
-    m_lineLayouts.slotEditDone(range.start().line(), range.start().line(), 0, m_textLayouts);
+    m_lineLayouts.slotEditDone(m_renderer, range.start().line(), range.start().line(), 0, m_textLayouts);
 }
 
 void KateLayoutCache::clear()
