@@ -13,19 +13,14 @@
 
 #include "katepartdebug.h"
 
-#include "katedocument.h"
-#include "katerenderer.h"
-
-KateLineLayout::KateLineLayout(KateRenderer &renderer)
-    : m_renderer(renderer)
-    , m_line(-1)
+KateLineLayout::KateLineLayout()
+    : m_line(-1)
     , m_virtualLine(-1)
 {
 }
 
 void KateLineLayout::clear()
 {
-    m_textLine.reset();
     m_line = -1;
     m_virtualLine = -1;
     shiftX = 0;
@@ -39,30 +34,15 @@ bool KateLineLayout::includesCursor(const KTextEditor::Cursor realCursor) const
     return realCursor.line() == line();
 }
 
-const Kate::TextLine &KateLineLayout::textLine(bool reloadForce) const
-{
-    if (reloadForce || !m_textLine) {
-        m_textLine.reset();
-        if (m_line >= 0 && m_line < m_renderer.doc()->lines()) {
-            m_textLine = usePlainTextLine ? m_renderer.doc()->plainKateTextLine(m_line) : m_renderer.doc()->kateTextLine(m_line);
-        }
-    }
-
-    Q_ASSERT(m_textLine);
-
-    return *m_textLine;
-}
-
 int KateLineLayout::line() const
 {
     return m_line;
 }
 
-void KateLineLayout::setLine(int line, int virtualLine)
+void KateLineLayout::setLine(Kate::TextFolding &folding, int line, int virtualLine)
 {
     m_line = line;
-    m_virtualLine = (virtualLine == -1) ? m_renderer.folding().lineToVisibleLine(line) : virtualLine;
-    m_textLine.reset();
+    m_virtualLine = (virtualLine == -1) ? folding.lineToVisibleLine(line) : virtualLine;
 }
 
 int KateLineLayout::virtualLine() const
@@ -75,18 +55,18 @@ void KateLineLayout::setVirtualLine(int virtualLine)
     m_virtualLine = virtualLine;
 }
 
-bool KateLineLayout::startsInvisibleBlock() const
+bool KateLineLayout::startsInvisibleBlock(Kate::TextFolding &folding) const
 {
     if (!isValid()) {
         return false;
     }
 
-    return (virtualLine() + 1) != m_renderer.folding().lineToVisibleLine(line() + 1);
+    return (virtualLine() + 1) != folding.lineToVisibleLine(line() + 1);
 }
 
 bool KateLineLayout::isValid() const
 {
-    return line() != -1 && layout().lineCount() > 0 && (textLine(), m_textLine);
+    return line() != -1 && layout().lineCount() > 0;
 }
 
 void KateLineLayout::endLayout()
@@ -125,11 +105,6 @@ KTextEditor::Cursor KateLineLayout::start() const
     return KTextEditor::Cursor(line(), 0);
 }
 
-int KateLineLayout::length() const
-{
-    return textLine().length();
-}
-
 int KateLineLayout::viewLineCount() const
 {
     return m_layout.lineCount();
@@ -162,15 +137,10 @@ int KateLineLayout::widthOfLastLine()
     return lastLine.width() + lastLine.xOffset();
 }
 
-bool KateLineLayout::isOutsideDocument() const
-{
-    return line() < 0 || line() >= m_renderer.doc()->lines();
-}
-
 void KateLineLayout::debugOutput() const
 {
-    qCDebug(LOG_KTE) << "KateLineLayout: " << this << " valid " << isValid() << " line " << line() << " length " << length() << " width " << width()
-                     << " viewLineCount " << viewLineCount();
+    qCDebug(LOG_KTE) << "KateLineLayout: " << this << " valid " << isValid() << " line " << line() << " width " << width() << " viewLineCount "
+                     << viewLineCount();
 }
 
 int KateLineLayout::viewLineForColumn(int column) const
