@@ -419,7 +419,7 @@ static bool rangeLessThanForRenderer(const Kate::TextRange *a, const Kate::TextR
     return a->start().toCursor() < b->start().toCursor();
 }
 
-QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::TextLine &textLine, int line, bool selectionsOnly) const
+QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::TextLine &textLine, int line, bool skipSelections) const
 {
     // limit number of attributes we can highlight in reasonable time
     const int limitOfRanges = 1024;
@@ -430,7 +430,7 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::Tex
 
     // Don't compute the highlighting if there isn't going to be any highlighting
     const auto &al = textLine.attributesList();
-    if (!(selectionsOnly || !al.empty() || !rangesWithAttributes.empty())) {
+    if (al.empty() && rangesWithAttributes.empty() && !m_view->selection()) {
         return QList<QTextLayout::FormatRange>();
     }
 
@@ -483,7 +483,7 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::Tex
     }
 
     // Add selection highlighting if we're creating the selection decorations
-    if ((m_view && showSelections() && m_view->selection()) || (m_view && m_view->blockSelection())) {
+    if (!skipSelections && ((m_view && showSelections() && m_view->selection()) || (m_view && m_view->blockSelection()))) {
         auto &currentRange = renderRanges.pushNewRange();
 
         // Set up the selection background attribute TODO: move this elsewhere, eg. into the config?
@@ -492,6 +492,7 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::Tex
             backgroundAttribute = KTextEditor::Attribute::Ptr(new KTextEditor::Attribute());
         }
 
+        backgroundAttribute->setBackground(config()->selectionColor());
         backgroundAttribute->setForeground(attribute(KSyntaxHighlighting::Theme::TextStyle::Normal)->selectedForeground().color());
 
         // Create a range for the current selection
@@ -508,23 +509,8 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::Tex
     }
 
     // Calculate the range which we need to iterate in order to get the highlighting for just this line
-    KTextEditor::Cursor currentPosition;
-    KTextEditor::Cursor endPosition;
-    if (m_view && selectionsOnly) {
-        if (m_view->blockSelection()) {
-            KTextEditor::Range subRange = m_doc->rangeOnLine(m_view->selectionRange(), line);
-            currentPosition = subRange.start();
-            endPosition = subRange.end();
-        } else {
-            KTextEditor::Range rangeNeeded = m_view->selectionRange() & KTextEditor::Range(line, 0, line + 1, 0);
-
-            currentPosition = qMax(KTextEditor::Cursor(line, 0), rangeNeeded.start());
-            endPosition = qMin(KTextEditor::Cursor(line + 1, 0), rangeNeeded.end());
-        }
-    } else {
-        currentPosition = KTextEditor::Cursor(line, 0);
-        endPosition = KTextEditor::Cursor(line + 1, 0);
-    }
+    KTextEditor::Cursor currentPosition = KTextEditor::Cursor(line, 0);
+    const KTextEditor::Cursor endPosition = KTextEditor::Cursor(line + 1, 0);
 
     // Background formats have lower priority so they get overriden by selection
     const KTextEditor::Range selectionRange = m_view->selectionRange();
@@ -559,8 +545,12 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine(const Kate::Tex
         KTextEditor::Attribute::Ptr a = renderRanges.generateAttribute();
         if (a) {
             fr.format = *a;
-            if (selectionRange.contains(currentPosition)) {
-                assignSelectionBrushesFromAttribute(fr, *a);
+            if (!skipSelections && selectionRange.contains(currentPosition)) {
+                if (!m_view->blockSelection()) {
+                    assignSelectionBrushesFromAttribute(fr, *a);
+                } else if (m_doc->rangeOnLine(selectionRange, line).containsColumn(currentPosition.column())) {
+                    assignSelectionBrushesFromAttribute(fr, *a);
+                }
             }
         }
 
@@ -589,26 +579,16 @@ void KateRenderer::assignSelectionBrushesFromAttribute(QTextLayout::FormatRange
     }
 }
 
-void KateRenderer::paintTextBackground(QPainter &paint,
-                                       KateLineLayout *layout,
-                                       const QList<QTextLayout::FormatRange> &selRanges,
-                                       const QBrush &brush,
-                                       int xStart) const
+void KateRenderer::paintTextBackground(QPainter &paint, KateLineLayout *layout, const QList<QTextLayout::FormatRange> &selRanges, int xStart) const
 {
     const bool rtl = layout->isRightToLeft();
 
     for (const auto &sel : selRanges) {
         const int s = sel.start;
         const int e = sel.start + sel.length;
-        QBrush br;
+        QBrush br = sel.format.background();
 
-        // Prefer using the brush supplied by user
-        if (brush != Qt::NoBrush) {
-            br = brush;
-        } else if (sel.format.background() != Qt::NoBrush) {
-            // Otherwise use the brush in format
-            br = sel.format.background();
-        } else {
+        if (br == Qt::NoBrush) {
             // skip if no brush to fill with
             continue;
         }
@@ -672,6 +652,7 @@ void KateRenderer::paintTextLine(QPainter &paint,
 
     // font data
     const QFontMetricsF &fm = m_fontMetrics;
+    const Kate::TextLine textLine = doc()->kateTextLine(range->line());
 
     int currentViewLine = -1;
     if (cursor && cursor->line() == range->line()) {
@@ -681,7 +662,7 @@ void KateRenderer::paintTextLine(QPainter &paint,
     paintTextLineBackground(paint, range, currentViewLine, xStart, xEnd);
 
     // Draws the dashed underline at the start of a folded block of text.
-    if (!(flags & SkipDrawFirstInvisibleLineUnderlined) && range->startsInvisibleBlock()) {
+    if (!(flags & SkipDrawFirstInvisibleLineUnderlined) && range->startsInvisibleBlock(folding())) {
         QPen pen(config()->foldingColor());
         pen.setCosmetic(true);
         pen.setStyle(Qt::DashLine);
@@ -717,34 +698,17 @@ void KateRenderer::paintTextLine(QPainter &paint,
             }
         }
 
-        if (range->length() > 0) {
+        if (textLine.length() > 0) {
             // We may have changed the pen, be absolutely sure it gets set back to
             // normal foreground color before drawing text for text that does not
             // set the pen color
             paint.setPen(attribute(KSyntaxHighlighting::Theme::TextStyle::Normal)->foreground().color());
+
+            // Draw text background
+            const auto decos = decorationsForLine(textLine, range->line(), /*skipSelections=*/!drawSelection);
+            paintTextBackground(paint, range, decos, xStart);
+
             // Draw the text :)
-
-            if (range->layout().textOption().textDirection() == Qt::RightToLeft) {
-                // If the text is RTL, we draw text background ourselves
-                auto decos = decorationsForLine(range->textLine(), range->line(), false);
-                auto sr = view()->selectionRange();
-                auto c = config()->selectionColor();
-                int line = range->line();
-                // Remove "selection" format from decorations
-                // "selection" will get painted below
-                decos.erase(std::remove_if(decos.begin(),
-                                           decos.end(),
-                                           [sr, c, line](const QTextLayout::FormatRange &fr) {
-                                               return sr.overlapsLine(line) && sr.overlapsColumn(fr.start) && fr.format.background().color() == c;
-                                           }),
-                            decos.end());
-                paintTextBackground(paint, range, decos, Qt::NoBrush, xStart);
-            }
-
-            if (drawSelection) {
-                QList<QTextLayout::FormatRange> additionalFormats = decorationsForLine(range->textLine(), range->line(), true);
-                paintTextBackground(paint, range, additionalFormats, config()->selectionColor(), xStart);
-            }
             range->layout().draw(&paint, QPoint(-xStart, 0), QList<QTextLayout::FormatRange>{}, textClipRect);
         }
 
@@ -766,7 +730,7 @@ void KateRenderer::paintTextLine(QPainter &paint,
             // Draw indent lines
             if (!m_printerFriendly && (indentLinesEnabled && i == 0)) {
                 const qreal w = spaceWidth();
-                const int lastIndentColumn = range->textLine().indentDepth(m_tabWidth);
+                const int lastIndentColumn = textLine.indentDepth(m_tabWidth);
                 for (int x = m_indentWidth; x < lastIndentColumn; x += m_indentWidth) {
                     auto xPos = x * w + 1 - xStart;
                     if (xPos >= 0) {
@@ -776,7 +740,7 @@ void KateRenderer::paintTextLine(QPainter &paint,
             }
 
             // draw an open box to mark non-breaking spaces
-            const QString &text = range->textLine().text();
+            const QString &text = textLine.text();
             int y = lineHeight() * i + m_fontAscent - fm.strikeOutPos();
             int nbSpaceIndex = text.indexOf(nbSpaceChar, line.lineLayout().xToCursor(xStart));
 
@@ -805,7 +769,7 @@ void KateRenderer::paintTextLine(QPainter &paint,
             // draw trailing spaces
             if (showSpaces() != KateDocumentConfig::None) {
                 int spaceIndex = line.endCol() - 1;
-                const int trailingPos = showSpaces() == KateDocumentConfig::All ? 0 : qMax(range->textLine().lastChar(), 0);
+                const int trailingPos = showSpaces() == KateDocumentConfig::All ? 0 : qMax(textLine.lastChar(), 0);
 
                 if (spaceIndex >= trailingPos) {
                     QVarLengthArray<int, 32> spacePositions;
@@ -970,9 +934,9 @@ void KateRenderer::paintTextLine(QPainter &paint,
             } else /* rtl + dynWordWrap == false */ {
                 // if text is rtl and dynamic wrap is false, the x offsets are in the opposite
                 // direction i.e., [0] == biggest offset, [1] = next
-                x = range->viewLine(viewLine).lineLayout().cursorToX(range->length() - column) - xStart;
+                x = range->viewLine(viewLine).lineLayout().cursorToX(textLine.length() - column) - xStart;
             }
-            int textLength = range->length();
+            int textLength = textLine.length();
             if (column == 0 || column < textLength) {
                 // If the note is inside text or at the beginning, then there is a hole in the text where the
                 // note should be painted and the cursor gets placed at the right side of it. So we have to
@@ -1022,7 +986,8 @@ void KateRenderer::paintCaret(KTextEditor::Cursor cursor, KateLineLayout *range,
         int caretWidth;
         int lineWidth = 2;
         QColor color;
-        QTextLine line = range->layout().lineForTextPosition(qMin(cursor.column(), range->length()));
+        const int lineLength = range->layout().text().length();
+        QTextLine line = range->layout().lineForTextPosition(qMin(cursor.column(), lineLength));
 
         // Determine the caret's style
         KTextEditor::caretStyles style = caretStyle();
@@ -1030,7 +995,7 @@ void KateRenderer::paintCaret(KTextEditor::Cursor cursor, KateLineLayout *range,
         // Make the caret the desired width
         if (style == KTextEditor::caretStyles::Line) {
             caretWidth = lineWidth;
-        } else if (line.isValid() && cursor.column() < range->length()) {
+        } else if (line.isValid() && cursor.column() < lineLength) {
             caretWidth = int(line.cursorToX(cursor.column() + 1) - line.cursorToX(cursor.column()));
             if (caretWidth < 0) {
                 caretWidth = -caretWidth;
@@ -1081,13 +1046,13 @@ void KateRenderer::paintCaret(KTextEditor::Cursor cursor, KateLineLayout *range,
             break;
         }
 
-        if (cursor.column() <= range->length()) {
+        if (cursor.column() <= lineLength) {
             // Ensure correct cursor placement for RTL text
             if (range->layout().textOption().textDirection() == Qt::RightToLeft) {
                 xStart += caretWidth;
             }
             qreal width = 0;
-            if (cursor.column() < range->length()) {
+            if (cursor.column() < lineLength) {
                 const auto inlineNotes = m_view->inlineNotes(range->line());
                 for (const auto &inlineNoteData : inlineNotes) {
                     KTextEditor::InlineNote inlineNote(inlineNoteData);
@@ -1225,12 +1190,10 @@ qreal KateRenderer::spaceWidth() const
     return m_fontMetrics.horizontalAdvance(spaceChar);
 }
 
-void KateRenderer::layoutLine(KateLineLayout *lineLayout, int maxwidth, bool cacheLayout) const
+void KateRenderer::layoutLine(Kate::TextLine textLine, KateLineLayout *lineLayout, int maxwidth, bool cacheLayout, bool skipSelections) const
 {
     // if maxwidth == -1 we have no wrap
 
-    Kate::TextLine textLine = lineLayout->textLine();
-
     QTextLayout &l = lineLayout->modifiableLayout();
     l.setText(textLine.text());
     l.setFont(m_font);
@@ -1277,30 +1240,10 @@ void KateRenderer::layoutLine(KateLineLayout *lineLayout, int maxwidth, bool cac
     l.setTextOption(opt);
 
     // Syntax highlighting, inbuilt and arbitrary
-    QList<QTextLayout::FormatRange> decorations = decorationsForLine(textLine, lineLayout->line());
-
-    // Qt works badly if you have RTL text and formats set on that text.
-    // It will shape the text according to the given format ranges which
-    // produces incorrect results as a letter in RTL can have a different
-    // shape depending upon where in the word it resides. The resulting output
-    // looks like: وقار vs وق ار, i.e, the ligature قا is broken into ق ا which
-    // is really bad for readability
-    if (opt.textDirection() == Qt::RightToLeft) {
-        // We can fix this to a large extent here by using QGlyphRun etc, but for now
-        // we only fix this for formats which have a background color and a foreground
-        // color that is same as "dsNormal". Reasoning is that, it is unlikely that RTL
-        // text will have a lot of cases where you have partially colored ligatures. BG
-        // formats are different, you can easily have a format that covers a ligature partially
-        // as a result of "Search" or "multiple cursor selection"
-        QColor c = view()->theme().textColor(KSyntaxHighlighting::Theme::Normal);
-        decorations.erase(std::remove_if(decorations.begin(),
-                                         decorations.end(),
-                                         [c](const QTextLayout::FormatRange &fr) {
-                                             return fr.format.hasProperty(QTextFormat::BackgroundBrush)
-                                                 && (fr.format.property(QTextFormat::ForegroundBrush).value<QBrush>().color() == c
-                                                     || fr.format.foreground() == Qt::NoBrush);
-                                         }),
-                          decorations.end());
+    QList<QTextLayout::FormatRange> decorations = decorationsForLine(textLine, lineLayout->line(), skipSelections);
+    // clear background, that is draw separately
+    for (auto &decoration : decorations) {
+        decoration.format.clearBackground();
     }
 
     int firstLineOffset = 0;
@@ -1491,9 +1434,9 @@ void KateRenderer::paintSelection(QPaintDevice *d, int startLine, int xStart, in
         }
 
         // compute layout WITHOUT cache to not poison it + render it
-        KateLineLayout lineLayout(*this);
-        lineLayout.setLine(line, -1);
-        layoutLine(&lineLayout, viewWidth, false /* no layout cache */);
+        KateLineLayout lineLayout;
+        lineLayout.setLine(folding(), line, -1);
+        layoutLine(doc()->kateTextLine(line), &lineLayout, viewWidth, false /* no layout cache */, /*skipSelections=*/true);
         KateRenderer::PaintTextLineFlags flags;
         flags.setFlag(KateRenderer::SkipDrawFirstInvisibleLineUnderlined);
         flags.setFlag(KateRenderer::SkipDrawLineSelection);
