@@ -239,8 +239,7 @@ public:
     /**
      * Change to a different font (soon to be font set?)
      */
-    void increaseFontSizes(qreal step = 1.0) const;
-    void decreaseFontSizes(qreal step = 1.0) const;
+    void addToFontSize(qreal step) const;
     void resetFontSizes() const;
 
     /**
@@ -277,7 +276,7 @@ public:
     /**
      * Text width & height calculation functions...
      */
-    void layoutLine(KateLineLayout *line, int maxwidth = -1, bool cacheLayout = false) const;
+    void layoutLine(Kate::TextLine textLine, KateLineLayout *line, int maxwidth = -1, bool cacheLayout = false, bool skipSelections = false) const;
 
     /**
      * This is a smaller QString::isRightToLeft(). It's also marked as internal to kate
@@ -296,7 +295,7 @@ public:
      *
      * \param selectionsOnly return decorations for selections and/or dynamic highlighting.
      */
-    QList<QTextLayout::FormatRange> decorationsForLine(const Kate::TextLine &textLine, int line, bool selectionsOnly = false) const;
+    QList<QTextLayout::FormatRange> decorationsForLine(const Kate::TextLine &textLine, int line, bool skipSelections = false) const;
 
     // Width calculators
     qreal spaceWidth() const;
@@ -304,9 +303,9 @@ public:
     /**
      * Returns the x position of cursor \p col on the line \p range.
      */
-    int cursorToX(const KateTextLayout &range, int col, bool returnPastLine = false) const;
+    qreal cursorToX(const KateTextLayout &range, int col, bool returnPastLine = false) const;
     /// \overload
-    int cursorToX(const KateTextLayout &range, const KTextEditor::Cursor pos, bool returnPastLine = false) const;
+    qreal cursorToX(const KateTextLayout &range, const KTextEditor::Cursor pos, bool returnPastLine = false) const;
 
     /**
      * Returns the real cursor which is occupied by the specified x value, or that closest to it.
@@ -381,7 +380,7 @@ public:
      */
     void paintTextLineBackground(QPainter &paint, KateLineLayout *layout, int currentViewLine, int xStart, int xEnd);
 
-    void paintTextBackground(QPainter &paint, KateLineLayout *layout, const QList<QTextLayout::FormatRange> &selRanges, const QBrush &br) const;
+    void paintTextBackground(QPainter &paint, KateLineLayout *layout, const QList<QTextLayout::FormatRange> &selRanges, int xStart) const;
 
     /**
      * This takes an in index, and returns all the attributes for it.
@@ -406,7 +405,7 @@ public:
      * @param xEnd              end pos on @a endLine in pixels
      * @param scale             the amount of scaling to apply. Default is 1.0, negative values are not supported
      */
-    void paintSelection(QPaintDevice *d, int startLine, int xStart, int endLine, int xEnd, qreal scale = 1.0);
+    void paintSelection(QPaintDevice *d, int startLine, int xStart, int endLine, int xEnd, int viewWidth, qreal scale = 1.0);
 
 private:
     /**
