@@ -19,7 +19,6 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonValue>
-#include <QMap>
 #include <QRegularExpression>
 #include <QStringList>
 #include <QUuid>
@@ -29,7 +28,9 @@
 #include <KLocalizedString>
 
 #include "katecmd.h"
+#include "katecommandlinescript.h"
 #include "kateglobal.h"
+#include "kateindentscript.h"
 #include "katepartdebug.h"
 
 KateScriptManager *KateScriptManager::m_instance = nullptr;
@@ -124,22 +125,23 @@ void KateScriptManager::collect()
         }
 
         QStringList list;
-        for (const QString &dir : qAsConst(dirs)) {
+        for (const QString &dir : std::as_const(dirs)) {
             const QStringList fileNames = QDir(dir).entryList({QStringLiteral("*.js")});
-            for (const QString &file : qAsConst(fileNames)) {
+            for (const QString &file : std::as_const(fileNames)) {
                 list.append(dir + QLatin1Char('/') + file);
             }
         }
 
         // iterate through the files and read info out of cache or file, no double loading of same scripts
         QSet<QString> unique;
-        for (const QString &fileName : qAsConst(list)) {
+        for (const QString &fileName : std::as_const(list)) {
             // get file basename
             const QString baseName = QFileInfo(fileName).baseName();
 
             // only load scripts once, even if multiple installed variants found!
-            if (unique.contains(baseName))
+            if (unique.contains(baseName)) {
                 continue;
+            }
 
             // remember the script
             unique.insert(baseName);
@@ -285,8 +287,12 @@ bool KateScriptManager::exec(KTextEditor::View *view, const QString &_cmd, QStri
     Q_UNUSED(view)
     Q_UNUSED(errorMsg)
 
-    QVector<QStringRef> args = _cmd.splitRef(QRegularExpression(QStringLiteral("\\s+")), Qt::SkipEmptyParts);
-    const QStringRef cmd = args.first();
+    const QList<QStringView> args = QStringView(_cmd).split(QRegularExpression(QStringLiteral("\\s+")), Qt::SkipEmptyParts);
+    if (args.isEmpty()) {
+        return false;
+    }
+
+    const auto cmd = args.first();
 
     if (cmd == QLatin1String("reload-scripts")) {
         reload();
@@ -307,3 +313,5 @@ bool KateScriptManager::help(KTextEditor::View *view, const QString &cmd, QStrin
 
     return false;
 }
+
+#include "moc_katescriptmanager.cpp"
