@@ -20,6 +20,7 @@
 
 #include <array>
 
+#include "katemodelinecompletion.h"
 #include "katetextfolding.h"
 #include "katetextrange.h"
 
@@ -131,6 +132,11 @@ private Q_SLOTS:
      */
     void pasteSelection();
 
+    /**
+     * Insert contents of selected file into the currently open file
+     */
+    void pasteFromFile();
+
     /**
      * Copy current selected stuff and paste previous content of clipboard as one operation.
      */
@@ -543,7 +549,7 @@ public:
 
     bool isLineRTL(int line) const;
 
-    QTextLayout *textLayout(const KTextEditor::Cursor pos) const;
+    const QTextLayout *textLayout(const KTextEditor::Cursor pos) const;
 
 public Q_SLOTS:
     void indent();
@@ -626,7 +632,7 @@ public Q_SLOTS:
      * space.
      * @see KateViewInternal::wordPrev
      */
-    void deleteWordLeft();
+    void deleteWordLeft(bool subword = false);
 
     /**
      * Remove the current selection. When nothing is selected the char right
@@ -641,7 +647,7 @@ public Q_SLOTS:
      * all trialling space removed.
      * @see KateViewInternal::wordNext
      */
-    void deleteWordRight();
+    void deleteWordRight(bool subword = false);
 
     /**
      * Transpose the characters left and right from the current cursor position
@@ -662,10 +668,10 @@ public Q_SLOTS:
     void shiftCursorLeft();
     void cursorRight();
     void shiftCursorRight();
-    void wordLeft();
-    void shiftWordLeft();
-    void wordRight();
-    void shiftWordRight();
+    void wordLeft(bool subword = false);
+    void shiftWordLeft(bool subword = false);
+    void wordRight(bool subword = false);
+    void shiftWordRight(bool subword = false);
     void markSelection();
     void home();
     void shiftHome();
@@ -955,6 +961,8 @@ private:
 public:
     bool insertTemplateInternal(const KTextEditor::Cursor insertPosition, const QString &templateString, const QString &script = QString());
 
+    bool evaluateScriptInternal(const QString &script, QVariant *result = nullptr);
+
     /**
      * Accessors to the bars...
      */
